@@ -54,6 +54,8 @@ export const Sensors = GObject.registerClass({
         this._addSettingChangedSignal('update-time', this._reconfigureNvidiaSmiProcess.bind(this));
         //this._addSettingChangedSignal('include-static-gpu-info', this._reconfigureNvidiaSmiProcess.bind(this));
 
+        this._gpu_drm_vendors = null;
+        this._gpu_drm_indices = null;
         this._nvidia_smi_process = null;
         this._nvidia_labels = [];
         this._bad_split_count = 0;
@@ -74,7 +76,7 @@ export const Sensors = GObject.registerClass({
 
     _refreshIPAddress(callback) {
         // check IP address
-        new FileModule.File('https://corecoding.com/vitals.php').read().then(contents => {
+        new FileModule.File('https://ipv4.corecoding.com').read().then(contents => {
             let obj = JSON.parse(contents);
             this._returnValue(callback, 'Public IP', obj['IPv4'], 'network', 'string');
         }).catch(err => { });
@@ -375,20 +377,20 @@ export const Sensors = GObject.registerClass({
     _queryBattery(callback) {
         let battery_slot = this._settings.get_int('battery-slot');
 
-        // addresses issue #161
-        let battery_key = 'BAT'; // BAT0, BAT1 and BAT2
-        if (battery_slot == 3) {
-            battery_slot = 'T';
-        } else if (battery_slot == 4) {
-            battery_key = 'CMB'; // CMB0
-            battery_slot = 0;
-        } else if (battery_slot == 5) {
-            battery_key = 'macsmc-battery'; // supports Asahi linux
-            battery_slot = '';
-        }
+        // create a mapping of indices to battery paths (from prefs.ui)
+        const BATTERY_PATHS = {
+            0: 'BAT0',
+            1: 'BAT1',
+            2: 'BAT2',
+            3: 'BATT',
+            4: 'CMB0',
+            5: 'CMB1',
+            6: 'CMB2',
+            7: 'macsmc-battery'
+        };
 
         // uevent has all necessary fields, no need to read individual files
-        let battery_path = '/sys/class/power_supply/' + battery_key + battery_slot + '/uevent';
+        let battery_path = '/sys/class/power_supply/' + BATTERY_PATHS[battery_slot] + '/uevent';
         new FileModule.File(battery_path).read("\n").then(lines => {
             let output = {};
             for (let line of lines) {
@@ -453,7 +455,7 @@ export const Sensors = GObject.registerClass({
                 this._returnValue(callback, 'Energy (now)', output['ENERGY_NOW'], 'battery', 'watt-hour');
             }
 
-            if ('ENERGY_FULL' in output && 'ENERGY_NOW' in output && 'POWER_NOW' in output && output['POWER_NOW'] > 0 && 'STATUS' in output && (output['STATUS'] == 'Charging' || output['STATUS'] == 'Discharging')) {
+            if ('ENERGY_FULL' in output && 'ENERGY_NOW' in output && 'POWER_NOW' in output && output['POWER_NOW'] !== 0 && 'STATUS' in output && (output['STATUS'] == 'Charging' || output['STATUS'] == 'Discharging')) {
 
                 let timeLeft = 0;
 
@@ -461,7 +463,7 @@ export const Sensors = GObject.registerClass({
                 if (output['STATUS'] == 'Charging') {
                     timeLeft = ((output['ENERGY_FULL'] - output['ENERGY_NOW']) / output['POWER_NOW']);
                 } else {
-                    timeLeft = (output['ENERGY_NOW'] / output['POWER_NOW']);
+                    timeLeft = (output['ENERGY_NOW'] / Math.abs(output['POWER_NOW']));
                 }
 
                 // don't process Infinity values
@@ -496,10 +498,16 @@ export const Sensors = GObject.registerClass({
 
     _queryGpu(callback) {
         if (!this._nvidia_smi_process) {
-            this._disableGpuLabels(callback);
-            return;
+            // no nvidia-smi, so we use sysfs DRM if any cards was discovered
+            if (!this._gpu_drm_indices){
+                this._disableGpuLabels(callback);
+                return;
+            } else {
+                this._readGpuDrm(callback);
+                return;
+            }
         }
-        
+
         this._nvidia_smi_process.read('\n').then(lines => {
             /// for debugging multi-gpu on systems with only one gpu
             /// duplicates the first gpu's data 3 times, for 4 total gpus
@@ -510,10 +518,9 @@ export const Sensors = GObject.registerClass({
             for (let i = 0; i < lines.length; i++) {
                 this._parseNvidiaSmiLine(callback, lines[i], i + 1, lines.length > 1);
             }
-            
 
-            // if we've already updated the static info during the last parse, then stop doing so. 
-            // this is so the _parseNvidiaSmiLine function won't return static info anymore 
+            // if we've already updated the static info during the last parse, then stop doing so.
+            // this is so the _parseNvidiaSmiLine function won't return static info anymore
             // and the nvidia-smi commmand won't be queried for static info either
             if(!this._nvidia_static_returned) {
                 this._nvidia_static_returned = true;
@@ -544,13 +551,13 @@ export const Sensors = GObject.registerClass({
         this._bad_split_count = 0;
 
         let [
-            label, 
+            label,
             fan_speed_pct,
-            temp_gpu, temp_mem, 
+            temp_gpu, temp_mem,
             mem_total, mem_used, mem_reserved, mem_free,
             util_gpu, util_mem, util_encoder, util_decoder,
             clock_gpu, clock_mem, clock_encode_decode,
-            power, power_avg, 
+            power, power_avg,
             link_gen_current, link_width_current
         ] = csv_split;
 
@@ -572,12 +579,9 @@ export const Sensors = GObject.registerClass({
             }
         }
 
-
         const typeName = 'gpu#' + gpuNum;
         const globalLabel = 'GPU' + (multiGpu ? ' ' + gpuNum : '');
-        
         const memTempValid = !isNaN(parseInt(temp_mem));
-        
 
         this._returnGpuValue(callback, 'Graphics', parseInt(util_gpu) * 0.01, typeName + '-group', 'percent');
 
@@ -628,6 +632,50 @@ export const Sensors = GObject.registerClass({
         this._returnStaticGpuValue(callback, 'Sub Device ID', staticInfo['sub_device_id'], typeName, 'string');
     }
 
+    _readGpuDrm(callback){
+        const multiGpu = this._gpu_drm_indices.length > 1;
+        const unit = this._settings.get_int('memory-measurement') ? 1000 : 1024;
+        for (let z = 0; z < this._gpu_drm_indices.length; z++ ) {
+            let i = this._gpu_drm_indices[z];
+            const typeName = 'gpu#' + i;
+            const vendor = this._gpu_drm_vendors[z];
+
+            // AMD
+            if(vendor === "0x1002") {
+                // read GPU usage and create group lebel for card
+                new FileModule.File('/sys/class/drm/card'+i+'/device/gpu_busy_percent').read().then(value => {
+                    // create group
+                    this._returnGpuValue(callback, 'Graphics', parseInt(value) * 0.01, typeName + '-group', 'percent');
+                    this._returnGpuValue(callback, 'Vendor', "AMD", typeName, 'string');
+                    this._returnGpuValue(callback, 'Usage', parseInt(value) * 0.01, typeName, 'percent');
+                }).catch(err => {
+                    // nothing to do, keep old value displayed
+                });
+                new FileModule.File('/sys/class/drm/card'+i+'/device/mem_info_vram_used').read().then(value => {
+                    this._returnGpuValue(callback, 'Memory Used', parseInt(value) / unit, typeName, 'memory');
+                }).catch(err => {
+                    // nothing to do, keep old value displayed
+                });
+                new FileModule.File('/sys/class/drm/card'+i+'/device/mem_info_vram_total').read().then(value => {
+                    this._returnGpuValue(callback, 'Memory Total', parseInt(value) / unit, typeName, 'memory');
+                }).catch(err => {
+                    // nothing to do, keep old value displayed
+                });
+            } else {
+                // for other vendors only show basic card info
+                let vendorName = null;
+                switch (vendor){
+                    case '0x10DE': vendorName = 'NVIDIA'; break; // should be never used as nvidia-smi should be preferred
+                    case '0x13B5': vendorName = 'ARM'; break;
+                    case '0x5143': vendorName = 'Qualcomm'; break;
+                    case '0x8086': vendorName = 'Intel'; break;
+                    default: vendorName = "Unknown " + vendor;
+                }
+                this._returnGpuValue(callback, 'Graphics', vendorName, typeName + '-group', 'string');
+            }
+        }
+    }
+
     _disableGpuLabels(callback) {
         for (let labelObj of this._nvidia_labels)
             this._returnValue(callback, labelObj.label, 'disabled', labelObj.type, labelObj.format);
@@ -635,7 +683,7 @@ export const Sensors = GObject.registerClass({
 
     _returnStaticGpuValue(callback, label, value, type, format) {
         //if we've already tried to return existing static info before or if the option isn't enabled, then do nothing.
-        if (this._nvidia_static_returned || !this._settings.get_boolean('include-static-gpu-info')) 
+        if (this._nvidia_static_returned || !this._settings.get_boolean('include-static-gpu-info'))
             return;
 
         //we don't need to disable static info labels, so just use ordinary returnValue function
@@ -645,7 +693,7 @@ export const Sensors = GObject.registerClass({
     _returnGpuValue(callback, label, value, type, format, display = true) {
         if(!display) return;
 
-        if(value === 'N/A' || value === '[N/A]' || isNaN(value)) return;
+        if(format !== "string" && (value === 'N/A' || value === '[N/A]' || isNaN(value))) return;
 
         let nvidiaLabel = {'label': label, 'type': type, 'format': format};
         if (!this._nvidia_labels.includes(nvidiaLabel))
@@ -748,6 +796,27 @@ export const Sensors = GObject.registerClass({
 
         // Launch nvidia-smi subprocess if nvidia querying is enabled
         this._reconfigureNvidiaSmiProcess();
+        this._discoverGpuDrm();
+    }
+
+    _discoverGpuDrm() {
+        // use DRM only if nvidia-smi is not used
+        if (this._settings.get_boolean('show-gpu') && this._nvidia_smi_process == null) {
+            // try to discover up to 10 cards starting from index 0
+            for(let i = 0; i < 10 ; i++){
+                new FileModule.File('/sys/class/drm/card'+i+'/device/vendor').read().then(value => {
+                    if(!this._gpu_drm_indices){
+                        this._gpu_drm_indices = [];
+                        this._gpu_drm_vendors = [];
+                    }
+                    this._gpu_drm_indices.push(i);
+                    this._gpu_drm_vendors.push(value);
+                }).catch(err => { });
+            }
+        } else {
+            this._gpu_drm_vendors = null;
+            this._gpu_drm_indices = null;
+        }
     }
 
     // The nvidia-smi subprocess will keep running and print new sensor data to stdout every
@@ -772,7 +841,7 @@ export const Sensors = GObject.registerClass({
     _reconfigureNvidiaSmiProcess() {
         if (this._settings.get_boolean('show-gpu')) {
             this._terminateNvidiaSmiProcess();
-            
+
             try {
                 let update_time = this._settings.get_int('update-time');
                 let query_interval = Math.max(update_time, 1);
@@ -786,13 +855,13 @@ export const Sensors = GObject.registerClass({
                     'clocks.gr,clocks.mem,clocks.video,' +
                     'power.draw.instant,power.draw.average,' +
                     'pcie.link.gen.gpucurrent,pcie.link.width.current,' +
-                    (!this._nvidia_static_returned && this._settings.get_boolean('include-static-gpu-info') ? 
+                    (!this._nvidia_static_returned && this._settings.get_boolean('include-static-gpu-info') ?
                         'temperature.gpu.tlimit,' +
                         'power.limit,' +
                         'pcie.link.gen.max,pcie.link.width.max,'   +
                         'addressing_mode,'+
                         'driver_version,vbios_version,serial,' +
-                        'pci.domain,pci.bus,pci.device,pci.device_id,pci.sub_device_id,' 
+                        'pci.domain,pci.bus,pci.device,pci.device_id,pci.sub_device_id,'
                     : ''),
                     '--format=csv,noheader,nounits',
                     '-l', query_interval.toString()
